/* ReSCAL - Engine entry
 *
 * Copyright (C) 2011
 *
 * Author: Olivier Rozier <rozier@ipgp.fr>
 *
 * This file is part of ReSCAL.
 *
 * ReSCAL is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include <stdio.h>
#include <locale.h>

#include "defs.h"
#include "macros_public.h"
#include "rescal.h"
//#include "monitor.h"
#include "interface.h"
#ifdef PARALLEL
#include "synchro.h"
#endif
#include "cellules.h"
#include "terre.h"
#include "surface.h"
#include "simul.h"
#include "modeles.h"
#include "doublets.h"
#include "transitions.h"
#include "param.h"
#include "trace.h"


#ifdef COLLISIONS
#include "collisions.h"
#endif

extern unsigned char opt_info, opt_nv;
extern unsigned long int iter;        // nombre d'iterations
extern double temps;                    // temps reel simule
#ifdef PARALLEL
extern int mode_par;    //mode parallele
#endif

//int etape_simulation = SIMUL_DEBUT; //etape courante de la simulation
int stop_simul = 0;                 //semaphore (pour le multithreading)
int fin_simul = 0;                  //semaphore (pour l'arret des callbacks)


void rescal_params()
{
  init_liste_params();
  params_simul();
  params_modele();
#ifdef COLLISIONS
  params_collisions();
#endif
}

void rescal_usage()
{
  printf("ReSCAL - %s model\n",MOD_NAME);
  printf("version %s (%s)\n",VER_NUM,VER_DAT);
  printf("size of cells : %lu byte(s)\n", sizeof(Cell));
#ifdef __x86_64
  printf("compiled for x86_64\n");
  /*
  printf("size of long: %ld\n", sizeof(long));
  printf("size of int: %ld\n", sizeof(int));
  printf("size of address: %ld\n", sizeof(char*));
  */
#endif

#ifdef USE_GD
  printf("linked with libgd\n");
#endif

#ifdef COLLISIONS
  printf("lattice gaz enabled\n");
#endif

  printf("\nusage: rescal PARAMETERS_FILE [OPTIONS]\n\n");
  rescal_params();
  param_usage();
}


void rescal_init(int argc, char ** argv)
{
  LogPrintf("\nReSCAL is starting\n");
  LogPrintf("version %s (%s)\n",VER_NUM,VER_DAT);

#ifdef NUM_MODE
  LogPrintf("format des valeurs numeriques : %s \n", NUM_MODE);
  setlocale(LC_NUMERIC, NUM_MODE);
#endif

  rescal_params();
  lecture_parametres(argv[1]);
  //init_signal();

#ifdef PARALLEL
  if (mode_par) synchro_init(argc, argv);
#endif

  cree_terre();
  calcule_couloir();
  compute_h_ceil();
  compute_h_floor();
#ifdef PARALLEL
  cree_tunnels();
#endif

  init_Ncel();
  init_simul();
  init_modele();
  init_transitions();
  init_db_pos();
#ifdef COLLISIONS
  init_collisions();
#endif

  if (opt_info){
    dump_cellules();
    dump_doublets();
    dump_transitions();
  }

#ifdef ALTI
  calcule_alti(ALTI, ALTI_MODE_BAS);
  calcule_normales();
#endif

#if defined(MODELE_DUN) && defined(COLLISIONS)
  //dump_cgv();
#endif

#ifdef SURRECTIONS
  //regul_niveau();
#endif

#if MODE_TRACE
  trace_init();
#endif

  LogPrintf("rescal_init: end\n");

}

int rescal(int n)
{
  static unsigned char start=0;
  unsigned char res=0;

  if (!start) {
#ifdef PARALLEL
    if (mode_par) synchro_start();
#endif
    //monitor_signal();
    LogPrintf("debut de la simulation ...\n");
    if (!opt_nv) push_status("running");
    start = 1;
  }

  res = 1;
  while ((!stop_simul) && (res)){
#if defined(TRACE_SRC) || defined(TRACE_AIRE) || defined(TRACE_PAR)
    res = trace_init_loop(0);
    if (!res) break;
#endif
    res = simul_terre(n, &stop_simul);
#if defined(TRACE_AIRE) || defined(TRACE_PAR)
    trace_end_loop();
    res = 1;
#endif
  }

  if (!res){
    LogPrintf("fin de la simulation\n");
    fin_simul = 1;
    if (!opt_nv) pop_status();
  }

  return (res);
}

/*

void rescal_quit()
{
  rescal_stop();
#ifdef TRACE_TRANS
  trace_trans_quit();
#endif
#ifdef TRACE3D_CEL
  trace3d_cel_quit();
#endif
  fin_db_pos();
  apocalypse();
}
*/

