/* ReSCAL - Stochastic simulation engine
 *
 * Copyright (C) 2011
 *
 * Author: Olivier Rozier <rozier@ipgp.fr>
 *
 * Code based on dissol program,
 * by Eduardo Sepulveda <edo@espci.fr>
 *
 * This file is part of ReSCAL.
 *
 * ReSCAL is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */



#include <stdio.h>
#include <stdlib.h>
#include <memory.h>
#include <math.h>

#include "defs.h"
#include "macros_public.h"
#include "param.h"
#include "cellules.h"
#include "terre.h"
#include "surface.h"
#include "doublets.h"
#ifdef PARALLEL
#include "synchro.h"
#endif
#include "transitions.h"
#include "simul.h"
#include "trace.h"
#ifdef COLLISIONS
#include "collisions.h"
#endif
#include "callbacks.h"
#include "interface.h"

extern unsigned char opt_info, opt_nv;
extern int H, L, P, HL, HLP;       // les dimensions de la terre
extern int LN, LS, LNS, HLN;    //couloir est-ouest (limite nord, limite sud, largeur nord-sud, ...)
extern Cell  *TE;	           // la 'terre'
extern int *db_pos[];                   // les tableaux contenant la position des doublets actifs
extern Doublet t_doub[MAX_DB];               // la description des doublets
extern int Ncel[];                       // nombre de cellules par type
extern int Ndb[];                        // nombre doublets actifs par type
extern int nb_trans_db, nb_trans_cel, nb_trans; // nombre de transitions de 2 cellules, d'1 cellule et au total
extern TransitionDb t_trans[];            // la description des transitions de doublets
extern TransitionCel t_trans_cel[];           // la description des transitions de cellules
extern LienTransDb   t_lien[];        // la description des liens de correlation entre transitions de 2 cellules
extern int nb_liens;             // nombre de liens entre transitions
extern const char *classes_db[];        // noms des classes de doublet
extern const char *etats[];             // les noms des types de cellules
#ifdef INFO_TRANS
extern int           cpt_lien[];      // compteur des correlations effectives
#endif
extern int           direction[];             // conversion classe de transition -> direction
#ifdef COLLISIONS
extern int col_iter;    //nombre de cycles de collisions
extern int CLEO;
extern char *nom_fic_mvt;
#ifdef CGV
extern float grdvc_max; //seuil max pour le controle de l'erosion en fonction du gradient de vitesse
extern float grdvc_min; //seuil min pour le controle de l'erosion en fonction du gradient de vitesse
#endif
#endif
extern float lambda_A;
#ifdef CELL_COLOR
extern float color_coef;         // coefficient associe a la callback check_color
#endif
#ifdef PARALLEL
extern int mode_par;    //mode parallele
#endif

#ifdef REFDB_PTR
extern RefDoublets *RefDB;       // references des cellules de la terre vers les doublets actifs
#else
extern RefDoublets_Type *RefDB_Type;       // references des cellules de la terre vers les doublets actifs
extern RefDoublets_Ind *RefDB_Ind;       // references des cellules de la terre vers les doublets actifs
#endif

#ifdef CGV
extern float cgv_coef;
#endif

unsigned long int iter=0, md_iter=0;  // nombre d'iterations (unites, milliards d'unites)
double temps=0.0;                  // temps reel simule
int cpt_trans_blocked[MAX_TRANSITIONS_DB];  //nombre de transitions blockees
double PII, PII_time;     //probabilités totales
double PI_ij[MAX_TRANSITIONS];                  // les PI_ij
double cum[MAX_TRANSITIONS];     //probabilites cumulees
int ava_mode = AVA_SYNC;  //mode d'avalanches
char *ava_mode_str = NULL;
int ava_trans = 0;        //transitions d'avalanches
int ava_norm = 0;       //avalanches avec normales
int ava_h_lim = 1; //10;	// hauteur limite avant avalanche
int ava_nb_cel_max = 1;  // nb de cellules qui tombent simultanement
float ava_delai = 0.0; //delai entre avalanches
float ava_angle = 0.0; //35.0; //angle d'avalanches (degres)
float ava_angle2 = 20.0; //angle d'avalanches (degres)
//#endif

#ifdef SURRECTIONS
float sur_delay = 0.0;  //10000.0;              // delai entre deux surrections successives (0 = pas de surrection)
int sur_mode = 0;                // type de surrection (uniforme ou locale)
char *sur_mode_str=NULL;
#endif

#ifdef ROTATIONS
int rot_mode = ROT_MODE_DISK;
float rot_angle = 0.0;
float rot_delay = 0.0;
#ifdef COLLISIONS
int rot_mvt_nb_cycles = 150;  //nombre de cycles de stabilisation apres rotation
#endif
char *rot_mode_str=NULL;
#endif

#ifdef COLLISIONS
int init_mvt_nb_cycles = -1;  //nombre de cycles de stabilisation du flux au depart
double delai_collisions = 0.0; //delai entre deux cycles de gaz sur reseau
#endif

void params_simul()
{
  //temps
  parametre("Time", "initial physical time - optional", &temps, PARAM_DOUBLE);

#ifdef AVALANCHES
  parametre("Ava_mode", "mode of avalanches (NONE|SYNC|TRANS|PROPAG, SYNC by default)", &ava_mode_str, PARAM_STRING);
  parametre("Ava_h_lim", "height limit for avalanches (1 by default)", &ava_h_lim, PARAM_INT);
  parametre("Ava_nb_cel", "number of cells falling together in SYNC mode (1 by default)", &ava_nb_cel_max, PARAM_INT);
  parametre("Ava_delay", "delay between avalanches", &ava_delai, PARAM_FLOAT);
  parametre("Ava_angle", "angle of avalanches (degrees) - optional", &ava_angle, PARAM_FLOAT);
#ifdef CELL_COLOR
  parametre("Ava_angle2", "angle of avalanches for colored grains (20 degrees by default)", &ava_angle2, PARAM_FLOAT);
#endif
#endif

#ifdef CELL_COLOR
  parametre("Color_coef", "control coefficient of colored grains - optional", &color_coef, PARAM_FLOAT);
#endif

#ifdef SURRECTIONS
  parametre("Sur_mode", "surrection mode (UNIFORM|CONE|TECTO)", &sur_mode_str, PARAM_STRING);
  parametre("Sur_delay", "surrection delay", &sur_delay, PARAM_FLOAT);
#endif

#ifdef ROTATIONS
  parametre("Rot_mode", "rotation mode (DISK|CYCLE) - optional", &rot_mode_str, PARAM_STRING);
  parametre("Rot_angle", "rotation angle - optional", &rot_angle, PARAM_FLOAT);
  parametre("Rot_delay", "rotation delay - optional", &rot_delay, PARAM_FLOAT);
#ifdef COLLISIONS
  parametre("Rot_ncycl", "number of flow cycles without transitions after rotation - optional", &rot_mvt_nb_cycles, PARAM_INT);
#endif
#endif

#ifdef COLLISIONS
  parametre("Init_ncycl", "initial number of flow cycles without transitions - optional", &init_mvt_nb_cycles, PARAM_INT);
  parametre("Lgca_delay", "physical time delay between flow cycles - optional", &delai_collisions, PARAM_DOUBLE);
#endif

#ifdef CGV
//  parametre("Grdvc", "critical shear stress for relationship between erosion rate and shear stress - optional", &grdvc, PARAM_FLOAT);
  parametre("Grdvc_max", "max shear stress value in the linear relationship between erosion rate and shear stress - optional", &grdvc_max, PARAM_FLOAT);
  parametre("Grdvc_min", "min shear stress value in the linear relationship between erosion rate and shear stress - optional", &grdvc_min, PARAM_FLOAT);
#endif
}

void simul_parse()
{
  if (ava_mode_str){
    if (!strcmp(ava_mode_str, "NONE")){
      ava_mode = AVA_NONE;
    }
    else if (!strcmp(ava_mode_str, "SYNC")){
      ava_mode = AVA_SYNC;
    }
    else if (!strcmp(ava_mode_str, "TRANS")){
      ava_mode = AVA_TRANS;
    }
    else if (!strcmp(ava_mode_str, "PROPAG")){
      ava_mode = AVA_PROPAG;
    }
    else{
      ErrPrintf("Incorrect value for Ava_mode : %s\n", ava_mode_str);
      exit(-2);
    }
  }
#ifdef SURRECTIONS
  if (sur_mode_str){
    if (!strcmp(sur_mode_str, "UNIFORM")){
      sur_mode = SUR_MODE_UNIFORM;
    }
    else if (!strcmp(sur_mode_str, "CONE")){
      sur_mode = SUR_MODE_CONE;
    }
    else if (!strcmp(sur_mode_str, "TECTO")){
      sur_mode = SUR_MODE_TECTO;
    }
  }
#endif
#ifdef ROTATIONS
  if (rot_mode_str){
    if (!strcmp(rot_mode_str, "DISK")){
      rot_mode = ROT_MODE_DISK;
    }
    else if (!strcmp(rot_mode_str, "CYCLE")){
      rot_mode = ROT_MODE_CYCL;
    }
  }
#endif
}

void init_simul()
{
  //params_simul();
  //params_modele();
  simul_parse();
  ava_trans = (ava_mode == AVA_TRANS) || (ava_mode == AVA_PROPAG);
  ava_norm = (ava_angle != 0);
}

void simul_lien_trans(int tr, int ix, int dir);
#ifdef COLLISIONS
void stabilisation_collisions(int nb_cyc);
#endif

// calcul des probabilites relatives de chaque transition
void calcule_prob()
{
  int i;
  double PI_ij_time, ccc;
  TransitionDb *ptr;
  TransitionCel *ptr_cel;

  //LogPrintf("calcul_prob: start\n");

  // transitions de doublet
  ptr = t_trans;
  for(i=0; i<nb_trans_db; i++, ptr++){
    if (ptr->regul) ptr->regul(&ptr->intensite);  //regulation de la transition
    PI_ij[i] = (double) Ndb[ptr->depart] *ptr->intensite;
#ifdef CGV
    //if (t_trans[i].check) PI_ij[i] *= cgv_coef;
#endif
    //LogPrintf("i=%d   t_trans[i].depart=%d   Ndb[t_trans[i].depart]=%d\n",i,t_trans[i].depart,Ndb[t_trans[i].depart]); //debug
  }

  // transitions de cellule
  ptr_cel = t_trans_cel;
  for(; i<nb_trans; i++,ptr_cel++){
    if (ptr_cel->regul) ptr_cel->regul(&ptr_cel->intensite); //regulation de la transition
    PI_ij[i] = (double) Ncel[ptr_cel->depart] * ptr_cel->intensite;
  }

  // calcul PII : probabilite qu'une transition quelconque se produise par unite de temps
  // calcul PII_time : idem avec correction de temps
  for(i=0, PII=0.0, ccc=0.0, PII_time = 0.0; i<nb_trans; i++){
    PII  += PI_ij[i];
    if (i<nb_trans_db){
      if (t_trans[i].time_mode == TIME_EVOL)
        PI_ij_time = PI_ij[i];
      else if (t_trans[i].time_mode == TIME_CORR){
#ifdef CGV
        PI_ij_time = PI_ij[i]*cgv_coef;
#else
        ErrPrintf("calcule_prob: no coefficient for time correction\n");
        exit(-1);
#endif
      }
      else
        PI_ij_time = 0;
    }
    else{
      PI_ij_time = PI_ij[i];
    }
/*#ifdef CGV
    //if ((i<nb_trans_db) && (t_trans[i].check != NULL)) PII_time  += PI_ij[i]*cgv_coef; else PII_time  += PI_ij[i];
    if ((i<nb_trans_db) && (t_trans[i].time_mode == TIME_CORR)) PII_time  += PI_ij[i]*cgv_coef; else PII_time  += PI_ij[i];
#endif*/
    cum[i] = PI_ij[i] + ccc;
    ccc    = cum[i];
    PII_time += PI_ij_time;
  }
/*#ifndef CGV
  PII_time = PII;
#endif*/

  //LogPrintf("PII = %f\n", PII);
  //return PII;
}

void simul_temps()
{
  double aleat, t;

    // le temps
  if (PII_time){
    aleat = drand48();
    if (!aleat) {LogPrintf("aleat = 0 !\n");  }
    t = -log(aleat)/PII_time;
    temps += t;
  }
  else{
    LogPrintf("simul_temps: PII_time = 0\n");
    /*if (PII){
      //on force le temps a evoluer
      aleat = drand48();
      if (!aleat) {LogPrintf("aleat = 0 !\n");  }
      t = -log(aleat)/PII;
      temps += t;
    }
    else{
      LogPrintf("simul_temps: PII = 0\n");
    }*/
    //LogPrintf("simul_temps() - cgv_coef = %f\n", cgv_coef);
    //int i;
    //for(i=0; i<nb_trans_db; i++) LogPrintf("[%d] : %d * %f\n", i, Ndb[t_trans[i].depart], t_trans[i].intensite);
  }
}

void simul_correl(int tr1, int ix, int ix2, int dir)
{
  static double cum2[MAX_LIENS];
  static int ln[MAX_LIENS], ind[MAX_LIENS];
  double ccc, aleat;
  int i, db, db2, n;
  int trans2_db_depart;
  //determination du doublet de depart
  db = type_doublet(ix, dir);
  db2 = type_doublet(ix2, dir);

  //construction du tableau des transitions correlees
  for(i=0, ccc=0.0, n=0; i<nb_liens; i++){
    if (t_lien[i].trans1 == tr1){
      trans2_db_depart = t_trans[t_lien[i].trans2].depart;
      if (t_lien[i].cel == 1){
        if (trans2_db_depart == db){
          ccc += t_lien[i].intensite;
          cum2[n] = ccc;
          ln[n] = i;
          ind[n] = ix;
          n++;
        }
      }
      else{
        if (trans2_db_depart == db2){
          ccc += t_lien[i].intensite;
          cum2[n] = ccc;
          ln[n] = i;
          ind[n] = ix2;
          n++;
        }
      }
    }
  }
  if (n > 0){ //test tableau non-vide ?
    //choix d'une transition selon les probabilites cumulees
    if (ccc > 1.0){
      ErrPrintf("ERROR : cumulated probabilites = %f > 1.0\n",ccc);
      exit(-1);
    }
    aleat = drand48();
    i=0;
    while ((i < n) && (aleat > cum2[i])) i++;
    if (i < n){
      //LogPrintf("Lien %d -> %d\n", tr1, ltr[i]);
      int tr2 = t_lien[ln[i]].trans2;
      //controle de la transition
      if (t_trans[tr2].check){
        if (!simul_check(tr2, ind[i], dir)) return; //transition bloquee
      }
      do_trans_db(tr2, ind[i], dir);
      iter++;
#ifdef INFO_TRANS
      cpt_lien[ln[i]]++;
#endif
      if (t_trans[tr2].LTR) simul_lien_trans(tr2, ind[i], dir);
    }
  }
}


void simul_lien_trans(int tr, int ix, int dir)
{
  int ix2;
  int ix_gauche, ix2_gauche;
  int ix_haut, ix2_haut;
  int ix_devant, ix2_devant;

  if (t_trans[tr].classe == VERTICAL){
    // PREMIERE TRANSITION VERTICALE
    ix2 = ix+L;

    // liens horizontaux, doublet droite
    simul_correl(tr, ix, ix2, DROITE);

    // liens horizontaux, doublet gauche
    ix_gauche = ix-1;
    ix2_gauche = ix2-1;
#ifdef CYCLAGE_HOR
    if (TE[ix_gauche].celltype == BORD){
      ix_gauche += (L-2);
      ix2_gauche += (L-2);
    }
#endif
    simul_correl(tr, ix_gauche, ix2_gauche, DROITE);

    // liens horizontaux, doublet derriere
    simul_correl(tr, ix, ix2, DERRIERE);

    // liens horizontaux, doublet devant
    ix_devant = ix-HL;
    ix2_devant = ix2-HL;
#ifdef CYCLAGE_HOR
    if (TE[ix_devant].celltype == BORD){
      ix_devant += (P-2)*HL;
      ix2_devant += (P-2)*HL;
    }
#endif
    simul_correl(tr, ix_devant, ix2_devant, DERRIERE);
  }
  else {
    // PREMIERE TRANSITION HORIZONTALE
    int flag_eo = (dir == DROITE/*EST_OUEST*/);
    ix2 = flag_eo ? ix+1 : ix+HL;
#ifdef CYCLAGE_HOR
    if (TE[ix2].celltype == BORD){
      ix2 -= flag_eo ? (L-2) : (P-2)*HL;
    }
#endif

    // liens verticaux, doublet bas
    simul_correl(tr, ix, ix2, BAS);

    // liens verticaux, doublet haut
    ix_haut = ix-L;
    ix2_haut = ix2-L;
    simul_correl(tr, ix_haut, ix2_haut, BAS);

    if (!flag_eo){
      // liens horizontaux, doublet droite
      simul_correl(tr, ix, ix2, DROITE);

      // liens horizontaux, doublet gauche
      ix_gauche = ix-1;
      ix2_gauche = ix2-1;
#ifdef CYCLAGE_HOR
      if (TE[ix_gauche].celltype == BORD){
        ix_gauche += (L-2);
        ix2_gauche += (L-2);
      }
#endif
      simul_correl(tr, ix_gauche, ix2_gauche, DROITE);
    }
    else{
      // liens horizontaux, doublet derriere
      simul_correl(tr, ix, ix2, DERRIERE);

      // liens horizontaux, doublet devant
      ix_devant = ix-HL;
      ix2_devant = ix2-HL;
#ifdef CYCLAGE_HOR
      if (TE[ix_devant].celltype == BORD){
        ix_devant += (P-2)*HL;
        ix2_devant += (P-2)*HL;
      }
#endif
      simul_correl(tr, ix_devant, ix2_devant, DERRIERE);
    }
  }
}


int simul_check(int tr, int ix, int dir)
{
  //int db_depart = t_trans[tr].depart;
  int chk = 0;
  DataCheck dc;

  //dc.trans = tr;
  dc.dir = dir;
  dc.cel = t_trans[tr].chk_cel;

  //Appel de la callback de controle (si elle est definie)
  if (t_trans[tr].chk_cel == 1){ // test sur la premiere cellule
    chk = t_trans[tr].check(ix, (void*)&dc);
  }
  else{ // test sur la deuxieme cellule
    int ix2;
    //if (t_doub[db_depart].classe == EST_OUEST){
    if (dir == DROITE){
      ix2 = ix+1;
#ifdef CYCLAGE_HOR
      if (TE[ix2].celltype == BORD) ix2 -= (L-2);
#endif
    }
    //else if (t_doub[db_depart].classe == NORD_SUD)
    else if (dir == DERRIERE){ //NORD_SUD
      ix2 = ix+HL;
#ifdef CYCLAGE_HOR
      if (TE[ix2].celltype == BORD) ix2 -= (P-2)*HL;
#endif
    }
    else { //BAS
      ix2 = ix+L;
    }
    chk = t_trans[tr].check(ix2, (void*)&dc);
  }

  if (!chk){
    cpt_trans_blocked[tr]++;
#ifdef TRACE_TRANS
    //trace_trans_blocked(tr, ix);
#endif
  }
  return chk;
}


#ifdef SURRECTIONS
void simul_sur()
{
  //static int seuil = HLP*10;  //10000000L;
  static float seuil = 0.0;

  if (sur_delay){
    if (!seuil){
      seuil = sur_delay;
      LogPrintf("delai de surrection = %f\n", sur_delay);
    }
    if (temps >= seuil){
        //regul_niveau();
        //if (calcule_hmin() > H/10) surrection_locale();
      //int nsur;
      //for(nsur=0; nsur<3; nsur++){
        //if (calcule_hmin() > 3) surrection();/*(type_sur == SUR_UNI) ? surrection() : surrection_locale();*/
        if (calcule_alti_max() < H-3) surrection();/*(type_sur == SUR_UNI) ? surrection() : surrection_locale();*/
        else { LogPrintf("surrection bloquee, sommet atteint !\n");  }
      //}
      seuil += sur_delay;
    }
  }
}
#endif

#ifdef ROTATIONS
//extern double lambda_F;
//extern double lambda_R;
//extern double lambda_D;
//extern double flux_eo;
extern int Ncel[];                       // nombre de cellules par type
extern float coef_injection; //regulation de l'injection de grain

void simul_rot()
{
  //const float PI = 3.14159;
  static float seuil = 0.0;
//  static char mode_rot = ROT_MODE_DISQUE;
  //static char mode_rot = ROT_MODE_CYCLAGE;
  static int cpt_rot = 0;

  // PI5
  static short lg_seq = 22; //2
//  static float delai_rot = 1000.0; //50.0; //25; //500.0;
  //static short seq_rot[] = {-1, 1}; //ROT_MODE_CYCLAGE
  static float seq_rot[] = {-1, 1, -1, 1, -1, 1, -1, 1, -1, 1, -5.0/6.0, 1, -1, 1, -1, 1, -1, 1, -1, 1, -1, 5.0/6.0}; //ROT_MODE_CYCLAGE
  //static short seq_del[] = {1, 1}; //ROT_MODE_CYCLAGE
  static short seq_del[] = {1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1}; //ROT_MODE_CYCLAGE
  //static short seq_rot[] = {1, 1, 1, 1, 1}; //ROT_MODE_DISQUE
  //static short seq_del[] = {1,1,1,1,1}; //ROT_MODE_DISQUE
//  static short seq_del[] = {2,2,3,3,2};
//  static short seq_del[] = {1,1,1,1,2};
//  static float seq_inj[] = {1.0e-1, 1.0e-1, 1.0e-1, 1.0e-1, 1.0e-1}; //ROT_MODE_DISQUE
  //static float seq_inj[] = {1.0, 0.0, 1.0, 1.0, 1.0}; //ROT_MODE_DISQUE
  static float seq_inj[] = {1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0}; //ROT_MODE_DISQUE
  //static float seq_gvc[] = {0.0, 200.0, 0.0, 0.0, 0.0}; //ROT_MODE_DISQUE

/*
  static short lg_seq = 2; //4  //longueur de sequence
  static short seq_rot[] = {1, -1};
  //static short seq_rot[] = {1, -1, -1, 1}; //essuie-glace
  //static short seq_rot[] = {1, 1, 1, 1, 1, 1}; //sequences de rotation
  static short seq_del[] = {1, 1}; //delais
  //static short seq_del[] = {1, 1, 1, 1}; //delais
  //static short seq_del[] = {4, 3, 3, 3, 3, 3}; //delais
  static float seq_inj[] = {1.0, 1.0};    //intensite injections
  //static float seq_inj[] = {1.0, 1.0, 1.0, 1.0};    //intensite injections
  //static float seq_inj[] = {1.0e-1, 1.0e-1, 1.0e-1, 1.0e-1, 0.0, 0.0};    //intensite injections
*/

  static int total_seq_rot = 0;
  int nseq;
  //static double lambda_F_debut = 0.0;

  if (!rot_delay) return;

  if (!seuil){
    seuil = rot_delay;
    LogPrintf("mode rotation = %s\n", (rot_mode == ROT_MODE_DISK) ? "table tournante" : "cyclage");
    LogPrintf("delai de rotation = %f\n", rot_delay);
#ifdef COLLISIONS
    LogPrintf("nombre de cycles de stabilisation apres rotation = %d\n", rot_mvt_nb_cycles);
    if (rot_mvt_nb_cycles < VSTEP_TIME) ErrPrintf("WARNING: stabilisation after rotation is too short ( Rot_ncycl = %d < VSTEP_TIME = %d )\n", rot_mvt_nb_cycles, VSTEP_TIME);
#endif
    //conditions_bords();
    //rotation(-3.14159*2.0/3.0);
    //lambda_F_debut = lambda_F;
    //lambda_F = 1.0 + (lambda_F_debut-1.0)*drand48();
    //init_transitions();
    //LogPrintf("lambda_F = %f\ttemps = %f\n", lambda_F, temps);

  }
  if (temps >= seuil){
    //translation
    /*int alea_dx = (drand48()-0.5)*L/5;
    int alea_dz = (drand48()-0.5)*L/5;
    translation(alea_dx, alea_dz);*/

    //rotation
    //short alea = 1 + (short)4.0*drand48();
    //short alea = (drand48()<0.5) ? 1 : 4;

    //rotation(alea*2.0*PI/5.0, mode_rot);
    //rotation(PI/2.0, mode_rot);
    nseq = cpt_rot-lg_seq*(int)(cpt_rot/lg_seq); //nseq = cpt_rot modulo lg_seq
    /*if ((rot_angle == floor(rot_angle)) && ((int)(rot_angle) % 90 == 0)){
      rotation90((int)(rot_angle/90));
    }
    else*/{
      rotation(/*seq_rot[nseq]**/2.0*PI*(rot_angle/360.0), rot_mode);
    }
    //rotation90(seq_rot[nseq]);
    cpt_rot++;
    total_seq_rot += seq_rot[nseq];
    //total_seq_rot=total_seq_rot%5;
    //if (total_seq_rot<0) total_seq_rot += lg_seq;
    //if (total_seq_rot>=lg_seq) total_seq_rot -= lg_seq;

    //coef_injection = (total_seq_rot <= 2)? 1.0 : 0.0;
    coef_injection = seq_inj[nseq];

#if 0
//#ifdef CGV
    //intensité du vent
    //grdvc_min = drand48()*500.0;
    grdvc_min = seq_gvc[nseq];
    grdvc_max = grdvc_min + 1000.0;
    LogPrintf("variation d'intensite du vent : grdvc_min = %f   grdvc_max = %f   temps = %f\n", grdvc_min, grdvc_max, temps);
#endif

    //t_lien[0].intensite = drand48();
    //t_lien[1].intensite = t_lien[0].intensite;
    //LogPrintf("intensite des liens 0 et 1 : %f\n", t_lien[0].intensite);

    //lambda_F = 2.0*lambda_F_debut*drand48();
    //lambda_F = 1.0 + (lambda_F_debut-1.0)*drand48();
    //lambda_F = 1.0;
    //lambda_R = 1.0;
    //lambda_D = 1.0;
    //flux_eo = 2.0;
    //init_transitions();
    //LogPrintf("lambda_F = %f\ttemps = %f\n", lambda_F, temps);
    //LogPrintf("lambda_F = %f\tlambda_R = %f\tlambda_D = %f\tflux_eo = %f\ttemps = %f\n", lambda_F, lambda_R, lambda_D, flux_eo, temps);
    //conditions_bords();
    //float delai_rot_alea = delai_rot*1.0*seq_del[nseq];
    //float delai_rot_alea = rot_delay*drand48();
    //float delai_rot_alea = delai_rot*0.5*(1.0+drand48());
    //seuil += delai_rot_alea;
    seuil += rot_delay;
    //seuil = 10000000L;

    //LogPrintf("cpt_rot = %d   delai_rot = %f   total_seq_rot = %d   coef_injection = %f\n", cpt_rot, rot_delay, total_seq_rot, coef_injection);
    LogPrintf("cpt_rot = %d\n", cpt_rot);

#if defined(GR) || defined(GRJ)
    LogPrintf("nombre de grains : %d\n", Ncel[GR] + Ncel[GRJ]);
#ifdef AVALANCHES
    if (ava_mode == AVA_SYNC){
      avalanches(GR, ava_h_lim, ava_nb_cel_max, ALTI_MODE_BAS);
    }
#endif
#endif

#ifdef COLLISIONS
    //stabilisation_collisions(VSTEP_TIME*3);
    stabilisation_collisions(rot_mvt_nb_cycles);
#endif
  }
}
#endif

#ifdef AVALANCHES
void simul_ava()
{
  //static float delai_ava = 1.0; //10.0;
  static float seuil_temps = 0.0;
  static unsigned long iter_0 = 0, md_iter_0;
  static unsigned long inter_iter_ava = 20000; //200000; //1000000; //nb min de transitions entre 2 avalanches
  if (!seuil_temps){
#ifdef MODELE_AVA
    //ava_delai = 0.1;
    inter_iter_ava = 1;
#endif
#ifdef DUMP_SIGMA
    inter_iter_ava = 1;
#endif
    //inter_iter_ava *= (float)(LNS/100.0);
    seuil_temps = ava_delai*drand48();
    //seuil_iter = inter_iter_ava;
    iter_0 = iter;
    md_iter_0 = md_iter;
    //LogPrintf("temps = %f\n", temps);
    LogPrintf("seuil temps avalanches = %f\n", seuil_temps);
    LogPrintf("delai avalanches = %f\n", ava_delai);
    LogPrintf("nb min iterations avant avalanches = %lu\n", inter_iter_ava);
    LogPrintf("ava_h_lim = %d\n", ava_h_lim);
    LogPrintf("ava_nb_cel_max = %d\n", ava_nb_cel_max);

        //avalanches(GR, ava_h_lim, ava_nb_cel_max, AVA_MODE_BAS);
        //avalanches_prob(GR);
  }
//#if !defined(DUMP_SIGMA) && !defined(DUMP_AUTOCORREL)
#if 0
  if ((!temps) || ((temps >= seuil_temps) && ((iter-iter_0 >= inter_iter_ava) || (md_iter-md_iter_0 > 0)))){
#else
  if ((!temps) || (temps >= seuil_temps)){
#endif
    //if (iter>960000) sleep(10); //debug
#ifdef DUMP_SIGMA
//    dump_sigma_alti();
#endif
    avalanches(ALTI, ava_h_lim, ava_nb_cel_max, ALTI_MODE_BAS);
    if (ava_angle){
      avalanches_norm(ALTI, ava_angle*PI/180.0, ava_nb_cel_max, ALTI_MODE_BAS);
    }

#ifdef DUMP_SIGMA
    dump_sigma_alti();
#endif
#ifdef DUMP_AUTOCORREL
    dump_autocorrel();
#endif
    seuil_temps = temps + ava_delai;
    iter_0 = iter;
    md_iter_0 = md_iter;
    /*seuil_iter = iter + inter_iter_ava;
    if (seuil_iter >= 1000000000L){
      seuil_iter -= 1000000000L;
    }*/
    //LogPrintf("iter=%ld\n", iter); //debug
    //if (iter>960000) sleep(10); //debug
  }
}

//avalanches avec un delai long (pour la stratification)
void simul_ava2()
{
  //static float delai_ava = 1.0; //10.0;
  static float seuil_temps = 0.0;
  static unsigned long iter_0 = 0, md_iter_0;
  static unsigned long inter_iter_ava = 2000000; //200000; //1000000; //nb min de transitions entre 2 avalanches
  if (!seuil_temps){
#ifdef MODELE_AVA
    //ava_delai = 10.0; // valeur 2D
    ava_delai = 100.0; // valeur 3D
    inter_iter_ava = 1;
#endif
    //inter_iter_ava *= (float)(LNS/100.0);
    seuil_temps = ava_delai*drand48();
    //seuil_iter = inter_iter_ava;
    iter_0 = iter;
    md_iter_0 = md_iter;
    //LogPrintf("temps = %f\n", temps);
    LogPrintf("simul_ava2\n");
    LogPrintf("seuil temps avalanches = %f\n", seuil_temps);
    LogPrintf("delai avalanches = %f\n", ava_delai);
    LogPrintf("nb min iterations avant avalanches = %lu\n", inter_iter_ava);
    LogPrintf("ava_h_lim = %d\n", 1);
    LogPrintf("ava_nb_cel_max = %d\n", ava_nb_cel_max);

        //avalanches(GR, ava_h_lim, ava_nb_cel_max, AVA_MODE_BAS);
        //avalanches_prob(GR);
  }
  if ((!temps) || ((temps >= seuil_temps) && ((iter-iter_0 >= inter_iter_ava) || (md_iter-md_iter_0 > 0)))){
    //if (iter>960000) sleep(10); //debug
      avalanches(ALTI, 1, ava_nb_cel_max, ALTI_MODE_BAS);
    if (ava_angle2){
      avalanches_norm(ALTI, ava_angle2*PI/180.0, ava_nb_cel_max, ALTI_MODE_BAS);
      //avalanches_norm(ALTI, 30*PI/180.0, ava_nb_cel_max, ALTI_MODE_BAS);
      //avalanches_norm(ALTI, ava_angle*PI/180.0, ava_nb_cel_max, ALTI_MODE_BAS); //appeler la fonction avalanches()
    }
      //LogPrintf("iter=%ld\n", iter); //debug
    //avalanches_prob(GR);
      seuil_temps = temps + ava_delai;
      iter_0 = iter;
      md_iter_0 = md_iter;
    /*seuil_iter = iter + inter_iter_ava;
      if (seuil_iter >= 1000000000L){
      seuil_iter -= 1000000000L;
    }*/
    //LogPrintf("iter=%ld\n", iter); //debug
    //if (iter>960000) sleep(10); //debug
    }
  }


//regulation temporelle des transitions d'avalanches : on bloque tout pendant ava_delai
void simul_ava_trans()
{
  //static float delai_ava = 1.0; //10.0;
  static float seuil_temps = 0.0;
  static float lambda_A_0 = 0.0;
  //static unsigned long iter_0 = 0, md_iter_0;
  //static unsigned long inter_iter_ava = 2000000; //200000; //1000000; //nb min de transitions entre 2 avalanches
  if (!seuil_temps){
#ifdef MODELE_AVA
    ava_delai = 200.0;
    lambda_A = 10;
    //inter_iter_ava = 1;
#endif
    //inter_iter_ava *= (float)(LNS/100.0);
    seuil_temps = ava_delai*drand48();
    lambda_A_0 = lambda_A;
    //seuil_iter = inter_iter_ava;
    //iter_0 = iter;
    //md_iter_0 = md_iter;
    //LogPrintf("temps = %f\n", temps);
    LogPrintf("simul_ava_trans() :\n");
    LogPrintf("seuil temps avalanches = %f\n", seuil_temps);
    LogPrintf("delai avalanches = %f\n", ava_delai);
    LogPrintf("lambda_A = %f\n", lambda_A_0);
    //LogPrintf("nb min iterations avant avalanches = %lu\n", inter_iter_ava);

  }
  if ((!temps) || (temps >= seuil_temps)){
  //if ((!temps) || ((temps >= seuil_temps) && ((iter-iter_0 >= inter_iter_ava) || (md_iter-md_iter_0 > 0)))){
    //if (iter>960000) sleep(10); //debug
    //LogPrintf("iter=%ld\n", iter); //debug
    //seuil_temps = temps + ava_delai;
    //lambda_A = lambda_A ? 0.0 : lambda_A_0;
    if (lambda_A){
      lambda_A = 0.0;
      seuil_temps = temps + ava_delai;
    }
    else{
      lambda_A = lambda_A_0;
      seuil_temps = temps + 1.0;
    }
    //LogPrintf("lambda_A = %f\n", lambda_A);
    //iter_0 = iter;
    //md_iter_0 = md_iter;
    /*seuil_iter = iter + inter_iter_ava;
    if (seuil_iter >= 1000000000L){
    seuil_iter -= 1000000000L;
  }*/
    //LogPrintf("iter=%ld\n", iter); //debug
    //if (iter>960000) sleep(10); //debug
  }
}

// regulation temporelle de l'angle limite
void simul_ava_angle()
{
  //static float delai_ava = 1.0; //10.0;
  static float seuil_temps = 0.0;
  static float ava_angle_dc = 0.0; //delai court
  static float ava_angle_dl = 0.0;  //delai long
  static float ava_angle2_dc = 0.0;
  static float ava_angle2_dl = 0.0;
#ifdef MODELE_AVA
// valeurs 2D
//  static float ava_delai_court = 50.0;
//  static float ava_delai_long = 200.0;
// valeurs 3D
  static float ava_delai_court = 100.0;
  static float ava_delai_long = 400.0;
#else
#ifndef COLLISIONS
// valeurs 2D
/*
  static float ava_delai_court = 400.0;
  static float ava_delai_long = 800.0;
*/
// valeurs 3D
  static float ava_delai_court = 100.0;
  static float ava_delai_long = 200.0;
#else
// valeurs 2D
/*
  static float ava_delai_court = 600.0;
  static float ava_delai_long = 1200.0;
*/
// valeurs 3D
  static float ava_delai_court = 150.0;
  static float ava_delai_long = 300.0;
#endif
#endif
  if (!seuil_temps){
    seuil_temps = ava_delai_court*drand48();
    ava_angle_dc = ava_angle;
    ava_angle_dl = atan(1.5*tan(ava_angle*PI/180.0))*180/PI; //pente * 1.5
    ava_angle2_dc = ava_angle2;
    ava_angle2_dl = atan(1.2*tan(ava_angle2*PI/180.0))*180/PI;
    LogPrintf("simul_ava_angle() :\n");
    LogPrintf("seuil temps avalanches = %f\n", seuil_temps);
    LogPrintf("delai court avalanches = %f\n", ava_delai_court);
    LogPrintf("delai long avalanches = %f\n", ava_delai_long);
    LogPrintf("ava_angle = %f (delai_court)| %f (delai_long)\n", ava_angle_dc, ava_angle_dl);
#ifdef CELL_COLOR
    LogPrintf("ava_angle2 = %f (delai_court)| %f (delai_long)\n", ava_angle2_dc, ava_angle2_dl);
#endif
    //LogPrintf("nb min iterations avant avalanches = %lu\n", inter_iter_ava);

  }
  if ((!temps) || (temps >= seuil_temps)){
    if (ava_angle == ava_angle_dc){
      ava_angle = ava_angle_dl;
      lambda_A /= 3;
      seuil_temps = temps + ava_delai_long;
    }
    else{
      ava_angle = ava_angle_dc;
      lambda_A *= 3;
      seuil_temps = temps + ava_delai_court;
    }
#ifdef CELL_COLOR
    if (ava_angle2 == ava_angle2_dc){
      //ava_angle2 *= 10; // pour bloquer les avalanches
      ava_angle2 = ava_angle2_dl; //pente * 3
      seuil_temps = temps + ava_delai_long;
    }
    else{
      ava_angle2 = ava_angle2_dc;
      seuil_temps = temps + ava_delai_court;
    }
#endif //CELL_COLOR
  }
}
#endif //AVALANCHES

#ifdef CGV
extern float lambda_F; //intensite erosion
extern float grdvc_max; //seuil max pour le controle de l'erosion en fonction du gradient de vitesse
extern float grdvc_min; //seuil min pour le controle de l'erosion en fonction du gradient de vitesse

void simul_intensite_vent()
{
  static float delai_vent_max = 200.0; //1000.0; //10.0;
  static float seuil_temps = 0.0;
  //static float pente_erosion_grdv = 0.0;
  //static float lambda_F_debut = 0.0;

  if (!seuil_temps){
    seuil_temps = delai_vent_max*0.5;
    //lambda_F_debut = lambda_F;
    //pente_erosion_grdv = lambda_F/(grdvc_max-grdvc_min);
    LogPrintf("parametres d'intensite du vent au debut : lambda_F = %f   grdvc_min = %f   grdvc_max = %f\n", lambda_F, grdvc_min, grdvc_max);
    //LogPrintf("pente_erosion_grdv = %f\n", pente_erosion_grdv);
    LogPrintf("delai max de variation du vent = %f\n", delai_vent_max);

  }
  if (temps >= seuil_temps){
    //lambda_F = 2.0*lambda_F_debut*drand48();
    //lambda_F = lambda_F_debut/(1.1-drand48());
    //grdvc_min = grdvc_max - lambda_F/pente_erosion_grdv;
    grdvc_min = drand48()*500.0;
    grdvc_max = grdvc_min + 1000.0;
    float delai_vent_alea = delai_vent_max*drand48();
    //if (delai_vent_max>=500){ //affichage de l'intensite seulement en frequence basse
      //LogPrintf("variation d'intensite du vent : lambda_F = %f   grdvc_min = %f   temps = %f\n", lambda_F, grdvc_min, temps);
      LogPrintf("variation d'intensite du vent : grdvc_min = %f   grdvc_max = %f   temps = %f\n", grdvc_min, grdvc_max, temps);
      LogPrintf("delai de variation du vent = %f\n", delai_vent_alea);

    //}
    seuil_temps += delai_vent_alea;
  }
}
#endif //CGV

#ifdef COLLISIONS
extern unsigned char opt_vss;
extern float *grdv;

void stabilisation_collisions(int nb_cyc)
{
  int i;
  char str_status[100];

  LogPrintf("stabilisation des collisions : %d cycles\n", nb_cyc);
  if (!opt_nv) push_status("flow stabilization (0%%)");
//#ifdef SF_REAL
#ifdef CGV
  calcule_alti(ALTI, ALTI_MODE_BAS);
  calcule_normales();
#endif
  for (i=0; i<nb_cyc; i++) {
    //int j;
    //for (j=0; j<100000/L; j++) calcule_alti(GR, ALTI_MODE_BAS); //pour ralentir ...
    do_collisions();
    do_propagations();
#ifdef CGV
    if (opt_vss && grdv) calcule_grad_vel();
#endif
    sprintf(str_status, "flow stabilization (%d%%)", (100*i)/nb_cyc);
    if (!opt_nv){
      pop_status();
      push_status(str_status);
      check_pause();
    }
  }
  LogPrintf("fin stabilisation des collisions\n");
  if (!opt_nv) pop_status();
}
#endif

#ifdef ALTI
extern unsigned char opt_ls;

void simul_norm_lum()
{
  static float seuil_calcule_normales = 0.0;
  static float delai_calcule_normales = 1.0;

  if ((opt_ls) && (temps >= seuil_calcule_normales)){
    //recalcul des normales pour le rendu avec eclairage
    calcule_normales();
    seuil_calcule_normales += delai_calcule_normales;
  }
}
#endif

int simul_terre(int nb_iter, int* stop)
{
  int       i, elt, tr, ii;
  double    ccc, aleat;
  int dir;
  int cpt_blk=0;
  int max_blk=HLP;
  char flag_evol_time;

#ifdef ALTI
  calcule_alti(ALTI, ALTI_MODE_BAS);
  simul_norm_lum();
#endif

#ifdef COLLISIONS
  double temps_collisions = temps;
  if (!delai_collisions){
#ifdef MODELE_DUN
    delai_collisions = 1.0/NB_MVT_EO; //1.0;
    //delai_collisions = 1.5/NB_MVT_EO; //1.0;
#else
    // calcul delai_collisions
    double lambda_max=0;
    for(i=0; i<nb_trans_db; i++){
      if (t_trans[i].intensite > lambda_max) lambda_max = t_trans[i].intensite;
    }
    delai_collisions = 1.0 / lambda_max;
#endif
  }
  LogPrintf("delai_collisions = %f\n", delai_collisions);
#endif //COLLISIONS

  /********* DEBUT du NOYAU *********/
#ifdef DUMP_SIGNATURE
  if (opt_info) dump_signature(0);
#endif

#ifdef DUMP_SIGMA
//  dump_sigma_alti();
#endif

#ifdef AVALANCHES
  if (ava_mode == AVA_SYNC){
    simul_ava();
#ifdef CELL_COLOR
    simul_ava2();
#endif
  }

  if (ava_trans && ava_delai){
    simul_ava_trans();
  }
#ifdef CELL_COLOR
  simul_ava_angle();
#endif
#endif //AVALANCHES


#ifdef COLLISIONS
//#if 0
//stabilisation des collisions
  if (init_mvt_nb_cycles<0) init_mvt_nb_cycles=2*CLEO; //default value
  else if (init_mvt_nb_cycles<VSTEP_TIME) init_mvt_nb_cycles=VSTEP_TIME; //min value
  if (nom_fic_mvt) init_mvt_nb_cycles=VSTEP_TIME;
  //rotation(2.0*3.14159/5.0, ROT_MODE_CYCLAGE);
  //stabilisation_collisions(nb_cyc0);
  stabilisation_collisions(init_mvt_nb_cycles);
#ifdef CGV
  //init_cgv();
  if (opt_info) dump_cgv();
#endif
  LogPrintf("max_blk = %d\n", max_blk);
#endif //COLLISIONS

  ii = 0;
  //memset(cpt_trans_blocked, 0, MAX_TRANSITIONS_DB*sizeof(int));
  ResetMemory(cpt_trans_blocked, int, MAX_TRANSITIONS_DB);


  //calcul des probabilites
  calcule_prob();

  do { /*iterations*/

    if (!opt_nv) check_pause();

#ifndef COLLISIONS
    //verifier s'il reste des doublets actifs ... pour eviter la floating exception si PII = 0
    //LogPrintf("PII=%f\n",PII);//debug
    if (PII == 0) break;
#endif

    //for (i=0; i<100000/L; i++) calcule_alti(GR, ALTI_MODE_BAS); //pour ralentir ...

#ifdef COLLISIONS
    if (cpt_blk > max_blk){
      temps = temps_collisions;
      LogPrintf("simul: too many blocked transitions (%d), time evolution forced (%f)\n", cpt_blk, temps);
      cpt_blk = 0;
    }

    if ((PII == 0) || (temps >= temps_collisions)){
      /*while (1)*/ {
#ifdef ALTI
        //calcule_alti(GR, ALTI_MODE_BAS);
        calcule_normales();
#endif
#ifdef CGV
        calcule_grad_vel();
        compute_coef_cgv();
#endif
        //LogPrintf("debut collisions...");
        do_collisions();
        //LogPrintf("propagations...");
        do_propagations();
        //LogPrintf("fin collisions...\n");
      }
      //temps_collisions += 0.01;
      temps_collisions += delai_collisions;

      calcule_prob();
      if (PII == 0) continue;
    }
#elif defined(ALTI)
    simul_norm_lum();
#endif


// la transition: tr
    tr = 0;
    aleat = drand48();
    ccc = PII * aleat;
    while (ccc >= cum[tr]) tr++;

    //LogPrintf("tr=%d\n",tr);//debug

    if (tr > nb_trans){
      ErrPrintf("ERROR: array overflow in t_trans (%d)\n", tr);
      exit(-1);
    }

    if (tr < nb_trans_db){
      // TRANSITION DE DOUBLET
      // le numero de doublet choisi: elt
      int db_depart = t_trans[tr].depart;
      elt = (int) floor( drand48() * Ndb[db_depart]);

      int ix = db_pos[db_depart][elt];
/*#ifdef TRACE
      if (ii>10) trace_test(ix);
#endif*/


      //calcul de la direction
      if (t_trans[tr].classe == HORIZONTAL){
#ifdef REFDB_PTR
        dir = (RefDB[ix][DROITE] == &db_pos[db_depart][elt]) ? DROITE : DERRIERE;
#else
        dir = ((RefDB_Type[ix][DROITE] == db_depart) && (RefDB_Ind[ix][DROITE] == elt)) ? DROITE : DERRIERE;
#endif
      }
      else
        dir = direction[t_trans[tr].classe];

      //controle de la transition
      if (t_trans[tr].check && !simul_check(tr, ix, dir)){
        //transition bloquee
        cpt_blk++;
        continue;
      }

      do_trans_db(tr, ix, dir);
      cpt_blk = 0;
      iter++; //LogPrintf("iter = %ld\n", iter);
      if (iter > 1000000000L){
        iter -= 1000000000L;
        md_iter++;
      }

#ifdef LIENS_TRANSITIONS
      //test enchainement de transitions
      if (t_trans[tr].LTR) simul_lien_trans(tr, ix, dir);
#endif

      flag_evol_time = (t_trans[tr].time_mode != TIME_NO_EVOL);
    }

    else{
      // TRANSITION DE CELLULE
      static int imax = 0;
      int ix;
      LogPrintf("tr = %d   aleat = %f   ccc = %f   PII = %f\n", tr, aleat, ccc, PII);

      tr -= nb_trans_db;
      i=0;
      do{ // on cherche une cellule du type cel_depart
        ix = (int) floor( drand48() * HLP);
        i++;
      }
      while (TE[ix].celltype != t_trans_cel[tr].depart);

      if (i>imax){
        imax = i;
        if (imax>100){
          LogPrintf ("WARNING: cell transition was difficult (%d tries)\n", imax);
        }
      }

      do_trans_cel(tr, ix);
      iter++; //LogPrintf("iter = %ld\n", iter);
      flag_evol_time = 1;
    }

    // le temps
    if (flag_evol_time) simul_temps();

    //fprintf(stdout,"%15.8lf %8d\n",tcum,NTOT[0]+NTOT[1]/3);fflush(stdout);
    //if (!Ndb[0] && !Ndb[1] && !Ndb[2] && !Ndb[3])
    //  printf("plus de blancs !\n");fflush(stdout);

    /*{
      //forcer une serie de dumps rapproches
      static int seuil = 0;
      if (iter >= seuil){
        monitor_signal();
        sleep(2);
        seuil += 500;
      }
      if (iter>=500000) exit(0);
    }*/

#ifdef SURRECTIONS
    simul_sur();
#endif

#ifdef ROTATIONS
    simul_rot();
#endif

#ifdef AVALANCHES
  if (ava_mode == AVA_SYNC){
    simul_ava();
#ifdef CELL_COLOR
    simul_ava2(); //avalanches basses frequences (stratification)
#endif
  }

  if (ava_trans && ava_delai){
    simul_ava_trans();  //activation des transitions d'avalanche
  }
#ifdef CELL_COLOR
  simul_ava_angle(); //regulation temporelle de l'angle limite
#endif
#endif //AVALANCHES

#ifdef CGV
    //simul_intensite_vent();
#endif

    ii++;

#ifdef DUMP_SIGNATURE
    if (opt_info &&  !(ii & 0x000fffff)){ //toutes les 1048576 transitions
    	//empreinte terrestre
        dump_signature(ii);
   }
#endif

#if defined(CELL_COLOR) && defined(MODELE_AVA)
   /*if (!(ii & 0x00007fff)){ //toutes les 65536 transitions
     init_color();
   }*/
#endif

#ifdef TRACE3D_CEL
    //if (!(ii & 0x000001fff)) trace3d_cel();  //toutes les 8192 transitions (au lieu de 4096)
    //if (!(ii & 0x00000000)) trace3d_cel();  //toutes les 256 transitions
    trace3d_cel();  //a chaque transition
#endif

#ifdef DUMP_SIGMA
    //if (!(ii & 0x000000fff)) dump_sigma_alti(); //toutes les 4096 transitions
    //if (!(ii & 0x0000000ff)) dump_sigma_alti(); //toutes les 256 transitions
#endif

#ifdef DUMP_AUTOCORREL
    //if (!(ii & 0x00000ffff)) dump_autocorrel();
#endif

#ifdef PARALLEL
    //if (mode_par && !(ii & 0xf)) {
    if (mode_par && !(ii & 0xff)) {
      //if (synchro_terre()) {
        //LogPrintf("pause synchro\n");
        while (synchro_terre());
        //LogPrintf("synchro ok\n");
      //}
    }
#endif

    //recalcul des probabilites
    calcule_prob();

  } while( ((!nb_iter) || (ii != nb_iter)) && (!*stop)); /* iterations */

  LogPrintf("end of simul\n");
	// si PII = 0.0 alors la terre est homogene...

  //monitor_signal();

#ifdef LISSAGE
  lissage_terre();
#endif

  //return (PII == 0);
  return (PII != 0);
}


void dump_temps(/*unsigned long int iter, unsigned long int md_iter, double temps*/)
{
  static int cpt = 0;
  static unsigned long int md_iter_0 = 0, iter_0 = 0;
  static double temps_0 = 0;
  long int delta_iter, delta_md_iter;
	FILE *fp;

  // fichier temps
  fp = fopen("TEMPS.txt","a");
  if (!cpt){
    fprintf(fp,"      \tnb trans. \tdelta trans. \ttemps    \tdelta temps");
#ifdef COLLISIONS
    fprintf(fp,"\tnb cyc.");
#endif
  }
  delta_iter = iter - iter_0;
  delta_md_iter = md_iter - md_iter_0;
  if ((delta_md_iter > 0) && (delta_iter < 0))
  {
    delta_iter += 1000000000;
    delta_md_iter--;
  }

  if (delta_md_iter)
    fprintf(fp,"\n%03d : \t%04lu%09lu \t%04lu%09lu \t%e \t%e", cpt++, md_iter, iter, delta_md_iter, delta_iter, temps, temps - temps_0);
  else
    fprintf(fp,"\n%03d : \t%04lu%09lu \t%09lu \t%e \t%e", cpt++, md_iter, iter, delta_iter, temps, temps - temps_0);
#ifdef COLLISIONS
  fprintf(fp,"\t%09d", col_iter);
#endif
  fclose(fp);

  md_iter_0 = md_iter;
  iter_0 = iter;
  temps_0 = temps;
}

void dump_check_info()
{
  static int cpt = 0;
  int i;
  char flag_trans[10];
  FILE *fp;

  fp = fopen("INFO_CHK.txt","a");

  if (!cpt){
    for(i=0; i<nb_trans_db; i++){
      *flag_trans = 0;
      if (t_trans[i].check){
        sprintf(flag_trans, " {%s}", (t_trans[i].chk_cel == 1) ? "c1" : "c2");
        fprintf(fp,"TR[%d] : %s, [%s, %s] -> [%s, %s], %.9f%s\n", i, classes_db[t_trans[i].classe],
              etats[t_doub[t_trans[i].depart].one], etats[t_doub[t_trans[i].depart].two],
              etats[t_doub[t_trans[i].arrivee].one], etats[t_doub[t_trans[i].arrivee].two],
              t_trans[i].intensite, flag_trans);
      }
    }
    fprintf(fp,"\nNombre de transitions bloquees\n");
    fprintf(fp,"%03d :",cpt);
    for (i=0; i<nb_trans_db; i++)
      if (t_trans[i].check) fprintf(fp," %10d", i);
  }
  else{
    fprintf(fp,"\n%03d :",cpt);
    for (i=0; i<nb_trans_db; i++){
      if (t_trans[i].check){
        fprintf(fp," %10d", cpt_trans_blocked[i]);
        cpt_trans_blocked[i] = 0;
      }
    }
  }

  fclose(fp);

  cpt++;
}

